<?php
/*
Plugin Name: Freightview for WooCommerce
Plugin URI: http://woothemes.com/woocommerce
Description: Provide real-time LTL quotes, from your carriers, in your WooCommerce shopping cart and when a customer is checking out.
Version: 1.5.4
Author: Freightview
Author URI: https://www.freightview.com
 * Text Domain: freightview
 * Domain Path: /languages
*/

if (!class_exists('FV')) {
  class FV
  {
    public static $active = true;

    public static function log($msg = '', $label = null)
    {
      if (!self::$active) {
        return;
      }
      $txt = date("Y-m-d H:i:s") . '  -  ';
      if ($label) {
        $txt .= $label . "\n";
      }
      $txt .= json_encode($msg);
      $upload_dir = wp_upload_dir();
      $dir = trailingslashit($upload_dir['basedir']) . DIRECTORY_SEPARATOR . 'wc-logs' . DIRECTORY_SEPARATOR;
      if (is_dir($dir)) {
        file_put_contents($dir . 'fv_log_' . date('Y-m-d') . '.txt', $txt . PHP_EOL, FILE_APPEND | LOCK_EX);
      }
    }
  }
}
FV::log(' =================================================== ');

define("WC_Freightview_ID", "WC_Freightview");

$fv_location_types = array(
  'business dock' => __('business dock', 'woocommerce-freightview'),
  'business no dock' => __('business no dock', 'woocommerce-freightview'),
  'residential' => __('residential', 'woocommerce-freightview'),
  'limited access' => __('limited access', 'woocommerce-freightview'),
  'trade show' => __('trade show', 'woocommerce-freightview'),
  'construction' => __('construction', 'woocommerce-freightview'),
  'farm' => __('farm', 'woocommerce-freightview')
);

$fv_shipping_classes = array('50', '55', '60', '65', '70', '77.5', '85', '92.5', '100', '110', '125', '150', '175', '200', '250', '300', '400', '500');

// debug helper
if (!class_exists('ppr')) {
  function ppr($var, $title = false)
  {
    if ($title) {
      ?>
        <fieldset class="fv_debug_fs fv">
        <legend onClick="(function(e, o){
                var l = jQuery(o);
                var p = l.parent().find('pre');
                p.toggle();
                l.find('span').text('<?php echo $title ?> [' + (p.is(':hidden') ? '-' : '+') + ']');
                })(event, this)">
            <span><?php echo $title ?> [-]</span>
        </legend>
      <?php
    }
    ?>
      <pre class="fv_debug" style="<?php echo $title ? 'display:none;' : '' ?>"><?php print_r($var); ?></pre>
    <?php
    if ($title) {
      ?>
        </fieldset>
      <?php
    }
  }
}

include 'sort.php';

/*
 * Check if WooCommerce is active
 */
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
  function freightview_shipping_method_init()
  {
    if (!class_exists('WC_Freightview')) {
      class WC_Freightview extends WC_Shipping_Method
      {
        private $log;

        private $admin_options_ran = false;

        private $freightview_endpoint = 'https://www.freightview.com/api/v1/rates';

        public $id = 'freightview'; // Id for your shipping method. Should be unique.
        public static $static_id = 'freightview';
        public $total_weight = 0;

        /**
         * Constructor for the shipping class.
         */
        public function __construct()
        {
          $this->id = WC_Freightview_ID;
          $this->method_title = __('Freightview', 'woocommerce-freightview');  // Title shown in admin
          $this->method_description = __('This is Freightview', 'woocommerce-freightview'); // Description shown in admin

          $this->init();
        }

        /**
         * Init settings.
         */
        public function init()
        {
          // Load the settings API
          $this->init_form_fields();
          $this->init_settings();

          // Define user set variables
          $this->enabled = $this->get_option('enabled');
          $this->title = $this->get_option('title');

          $this->zip_origin = $this->get_option('zip_origin');
          $this->origin_type = $this->get_option('origin_type');
          $this->origin_liftgate = $this->get_option('origin_liftgate');
          $this->origin_inside = $this->get_option('origin_inside');
          $this->customer_selects_accessorials = $this->get_option('customer_selects_accessorials');
          $this->api_key = $this->get_option('api_key');
          $this->rate_desc = $this->get_option('rate_desc');
          $this->max_rates = $this->get_option('max_rates');

          $this->err_missing_weight = $this->get_option('err_missing_weight');
          $this->display_debug = $this->get_option('display_debug');
          $this->debug = $this->get_option('debug');
          $this->show_errors = $this->get_option('show_errors');

          // Logger.
          if (class_exists('WC_Logger')) {
            $this->log = new WC_Logger();
          } else {
            $this->log = $this->woocommerce_method()->logger();
          }

          // Save settings in admin if you have any defined
          add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
        }

        public function is_available($package)
        {
          global $fv_shipping_classes;
          $total_weight = 0;
          $allValidShippingClasses = true;

          foreach ($package['contents'] as $key => $item) {
            $total_weight = $total_weight + ($item['data']->get_weight() * $item['quantity']);
            if ($item['data']->get_shipping_class() !== null && $item['data']->get_shipping_class() !== "") {
              $shipping_class = (float)str_replace('-', '.', str_replace('class-', '', $item['data']->get_shipping_class()));
              if (!in_array($shipping_class, $fv_shipping_classes) || $shipping_class === null) {
                $allValidShippingClasses = false;
              }
            }
          }

          $this->total_weight = $total_weight;

          if (!$allValidShippingClasses) {
            return false;
          }

          $minimum_weight = 80;

          if (is_numeric($this->get_option('minimum_weight'))) {
            $minimum_weight = $this->get_option('minimum_weight');
          }

          if ($minimum_weight > $total_weight) {
            return false;
          }

          return parent::is_available($package);
        }

        public function isAjax()
        {
          if (defined('DOING_AJAX') && DOING_AJAX) {
            return true;
          } else {
            return false;
          }
        }

        /**
         * calculate_shipping function.
         *
         * @param mixed $package
         */
        public function calculate_shipping($package = array())
        {
          if ($this->api_key == '') {
            echo '<div class="fv_alert_error">'
              . '<strong>' . __('Freightview Shipping: Unauthorized', 'woocommerce-freightview') . '</strong>'
              . '<br/>'
              . __('Please double check the API key in the Freightview section of WooCommerce Shipping.', 'woocommerce-freightview')
              . '</div>';

            return false;
          }

          if ($this->zip_origin == '') {
            echo '<div class="fv_alert_error">'
              . '<strong>' . __('Freightview Shipping: No Origin Zip Code', 'woocommerce-freightview') . '</strong>'
              . '<br/>'
              . __('Please set your origin zip code in the Freightview section of WooCommerce Shipping.', 'woocommerce-freightview')
              . '</div>';

            return false;
          }

          if ($package['destination']['postcode'] == '') {
            echo '<div class="fv_alert_info">'
              . __('To calculate shipping, please click on the "Calculate Shipping" button below.', 'woocommerce-freightview')
              . '</div>';

            return false;
          }

          $query = array(
            'pickupDate' => date('Y-m-d'),
            'originPostalCode' => $this->zip_origin,
            'originType' => $this->origin_type,
            'destPostalCode' => $package['destination']['postcode'],
            'destType' => isset($package['destination']['destination_type']) ? $package['destination']['destination_type'] : 'business dock',
            'items' => array(),
            'charges' => array(),
          );

          if ($this->origin_liftgate == 1) {
            $query['charges'][] = 'liftgate pickup';
          }

          if ($this->origin_inside == 1) {
            $query['charges'][] = 'inside pickup';
          }

          if ($this->get_option('customer_selects_accessorials') === "yes") {
            if (isset($package['destination']['destination_liftgate']) and $package['destination']['destination_liftgate'] == 1) {
              $query['charges'][] = 'liftgate delivery';
            }
            if (isset($package['destination']['destination_inside']) and $package['destination']['destination_inside'] == 1) {
              $query['charges'][] = 'inside delivery';
            }
          } else {
            $query['destType'] = $this->get_option('destination_type') !== null ? $this->get_option('destination_type') : 'business dock';
            if ($this->get_option('destination_liftgate') === "yes") {
              $query['charges'][] = 'liftgate delivery';
            }
            if ($this->get_option('destination_inside') === "yes") {
              $query['charges'][] = 'inside delivery';
            }
          }

          if (count($query['charges']) < 1) {
            unset($query['charges']);
          }

          $product_array = array();

          foreach ($package['contents'] as $key => $contents) {
            $quantity = $contents['quantity'];
            $product = $contents['data'];

            if ($quantity > 0 && $product->needs_shipping()) {

              $package = get_post_meta($product->get_id(), '_fv_wc_product_shipping_package', true);
              $package = $package ? $package : freightview_woocommerce_default_shipping_package();

              $product_array[] = $product;
              $query['items'][] = array(
                'freightClass' => $product->get_shipping_class() != '' ? (float)str_replace('-', '.', str_replace('class-', '', $product->get_shipping_class())) : "",
                'weight' => (int)($product->get_weight() * $quantity),
                'length' => (int)$product->get_length(),
                'width' => (int)$product->get_width(),
                'height' => (int)$product->get_height(),
                'pieces' => (int)$quantity,
                'package' => (string)$package
              );

            }
          }

          if (!$this->isAjax() and $this->display_debug == 'yes') {
            ppr($query, __('Query Parameters', 'woocommerce-freightview'));
          }

          if ($this->debug == 'yes') {
            $this->log->add('freightview', __('Query:', 'woocommerce-freightview') . ' ' . print_r($query, true));
          }

          $result = $this->request_estimate($query);

          if ($this->debug == 'yes') {
            $this->log->add('freightview', __('Result:', 'woocommerce-freightview') . ' ' . print_r($result, true));
          }

          if ($result === false or $result == 'Not Found') {
            echo '<div class="fv_alert_error">'
              . __('Ack!  Looks like we are having trouble requesting a shipping estimate from', 'woocommerce-freightview')
              . ' <a href="https://www.freightview.com">www.freightview.com</a>.<br/><br/>'
              . __('Please try again later or contact the website administrator.', 'woocommerce-freightview')
              . '</div>';

            return false;
          }

          if ($result == 'Unauthorized') {
            echo '<div class="fv_alert_error">'
              . '<strong>' . __('Freightview Shipping: Unauthorized', 'woocommerce-freightview') . '</strong>'
              . '<br/>'
              . __('Please double check the API key in the Freightview section of WooCommerce Shipping.', 'woocommerce-freightview')
              . '</div>';

            return false;
          }

          $results = json_decode($result);
          if (!$this->isAjax() and $this->display_debug == 'yes') {
            ppr($results, __('Results', 'woocommerce-freightview'));
          }
          if (!$this->isAjax() and $this->show_errors == 'yes' and $results->error and $results->error != '') {
            echo '<div class="fv_alert_error">';
            // missing weight
            preg_match_all("~item\.weight \((\d+)\)~i", $results->message, $matches);
            if (isset($matches[1]) and count($matches[1]) > 0) {
              echo $this->err_missing_weight . ' ';
              $temp = array();
              foreach ($matches[1] as $product_index) {
                $temp[] = $product_array[$product_index]->get_title();
              }
              echo implode(', ', $temp);
            }
            echo '</div>';
          }

          $rates = array();

          if ($results->rates and count($results->rates) > 0) {
            foreach ($results->rates as $rate) {
              if (strtolower($rate->status) == 'ok') {
                $rates[] = $rate;
              }
            }
          }

          if (count($rates) > 0) {
            if (!$this->isAjax() and $rates[0]->error and $rates[0]->error != '') {
              echo '<div class="fv_alert_error">' . $rates[0]->error . '</div>';
            } else {
              if (is_array($rates)) {
                $byTotal = function ($v) {
                  return $v->total == 0 ? 999999 : $v->total;
                };
                $rates = Sort::comparison($rates, array($byTotal), array(Sort::ASC));
              }

              for ($rate_index = 0; $rate_index < count($rates); ++$rate_index) {
                if (count($this->rates) < $this->max_rates) {
                  $result_rate = $rates[$rate_index];
                  $service_option = $result_rate->serviceOption;

                  if (!isset($result_rate->days) || $result_rate->days === 0 || $result_rate->days === "") {
                    continue;
                  }

                  if (is_numeric($service_option)) {
                    $service_option = date("g a", strtotime($result_rate->serviceOption . ":00"));
                  } elseif (!empty($result_rate->serviceOption)) {
                    $service_option = $result_rate->serviceOption;
                  } else {
                    $service_option = "NOSERVICEOPTION";
                  }

                  $title = str_replace(array(
                    '[carrierCode]',
                    '[providerCode]',
                    '[carrier]',
                    '[provider]',
                    '[paymentTerms]',
                    '[serviceType]',
                    '[serviceOption]',
                    '[days]',
                    '[interline]'
                  ), array(
                    $result_rate->carrierCode,
                    $result_rate->providerCode,
                    $result_rate->carrier,
                    (trim($result_rate->provider) != '' ? ' via ' . $result_rate->provider : ''),
                    $result_rate->paymentTerms,
                    $result_rate->serviceType,
                    $service_option,
                    ((int)$result_rate->days > 1 ? $result_rate->days . ' ' . __('days') : $result_rate->days . ' ' . __('day')),
                    $result_rate->interline
                  ), $this->rate_desc);

                  if (strpos($title, " NOSERVICEOPTION") !== false) {
                    $title = str_replace(" NOSERVICEOPTION", "", $title);
                  }
                  if (strpos($title, "NOSERVICEOPTION ") !== false) {
                    $title = str_replace("NOSERVICEOPTION ", "", $title);
                  }

                  $title = preg_replace('/\s+/', ' ', $title);

                  $provider_code = ($result_rate->providerCode ? $result_rate->providerCode : $result_rate->carrierCode);
                  $service_type = $result_rate->serviceType;
                  $service_option = $result_rate->serviceOption;
                  $days = $result_rate->days;

                  $rate = array(
                    'id' => $this->id . '_' . (str_replace(' ', '-', $title . '_' . $result_rate->serviceType)),//$this->id . '_' . $results->id . '_' . $result_rate->id,
                    'label' => $title,
                    'cost' => $result_rate->total,
                    'calc_tax' => 'per_item'
                  );

                  // Register the rate
                  $this->add_rate($rate);
                }
              }
            }
          }
        }

        /**
         * Admin options fields.
         */
        public function init_form_fields()
        {
          global $fv_location_types, $fv_shipping_classes;
          $this->form_fields = array(
            'api_key' => array(
              'title' => __('API key', 'woocommerce-freightview'),
              'type' => 'text',
              'description' => __('Your Freightview API Key.', 'woocommerce-freightview'),
              'desc_tip' => true
            ),
            'enabled' => array(
              'title' => __('Enable/disable', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Enable this shipping method', 'woocommerce-freightview'),
              'default' => 'no'
            ),
            'display_debug' => array(
              'title' => __('Display debug messages', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Enable debugging', 'woocommerce-freightview'),
              'default' => 'no'
            ),
            'show_errors' => array(
              'title' => __('Show API request errors', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Display errors from the API instead of silently failing', 'woocommerce-freightview'),
              'default' => 'yes'
            ),
            'title' => array(
              'title' => __('Title', 'woocommerce-freightview'),
              'type' => 'text',
              'description' => __('This controls the title which the user sees during checkout.', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => __('Freightview', 'woocommerce-freightview')
            ),
            'zip_origin' => array(
              'title' => __('Origin zip code', 'woocommerce-freightview'),
              'type' => 'text',
              'description' => __('Zip Code from where the requests are sent.', 'woocommerce-freightview'),
              'desc_tip' => true
            ),
            'origin_type' => array(
              'title' => __('Origin pickup type', 'woocommerce-freightview'),
              'type' => 'select',
              'description' => __('Pickup type from where the requests are sent.', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => 'business dock',
              'options' => $fv_location_types
            ),
            'origin_liftgate' => array(
              'title' => __('Origin lift gate', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Lift gate required for pickup?', 'woocommerce-freightview'),
              'default' => 'no'
            ),
            'origin_inside' => array(
              'title' => __('Inside pickup', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Inside pickup required?', 'woocommerce-freightview'),
              'default' => 'no'
            ),
            'customer_selects_accessorials' => array(
              'title' => __('Let customers select accessorials', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Let customers select accessorials?', 'woocommerce-freightview'),
              'default' => 'yes'
            ),
            'destination_type' => array(
              'title' => __('Destination delivery type', 'woocommerce-freightview'),
              'type' => 'select',
              'description' => __('Use this delivery type for all customers.', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => 'business dock',
              'options' => $fv_location_types
            ),
            'destination_liftgate' => array(
              'title' => __('Destination lift gate', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Lift gate required for destination?', 'woocommerce-freightview'),
              'default' => 'no'
            ),
            'destination_inside' => array(
              'title' => __('Inside delivery', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Inside delivery required?', 'woocommerce-freightview'),
              'default' => 'no'
            ),
            'max_rates' => array(
              'title' => __('Max rates', 'woocommerce-freightview'),
              'type' => 'integer',
              'description' => __('Maximum number of rates to display.', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => 5
            ),
            'minimum_weight' => array(
              'title' => __('Minimum weight', 'woocommerce-freightview'),
              'type' => 'integer',
              'description' => __('Only use this shipping method for shipments over this value (lbs).', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => 80
            ),
            'default_package_type' => array(
              'title' => __('Default package type', 'woocommerce-freightview'),
              'type' => 'select',
              'description' => __('Set the default package type for shipping.', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => freightview_woocommerce_default_shipping_package(),
              'options' => freightview_woocommerce_shipping_packages()
            ),
            'rate_desc' => array(
              'title' => __('Rate description', 'woocommerce-freightview'),
              'type' => 'text',
              'description' => __('This is how the rate description is displayed.<br/><small>Available terms: [carrierCode], [providerCode], [carrier], [provider], [paymentTerms], [serviceType], [serviceOption], [days], [interline]</small>', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => __('[carrier] [provider] ([days] [serviceType] [serviceOption])', 'woocommerce-freightview')
            ),
            'debug' => array(
              'title' => __('Debug log', 'woocommerce-freightview'),
              'type' => 'checkbox',
              'label' => __('Enable logging', 'woocommerce-freightview'),
              'default' => 'no',
              'description' => sprintf(__('Log quote requests (%s).', 'woocommerce-freightview'), '<code>woocommerce/logs/freightview-' . sanitize_file_name(wp_hash('freightview')) . '.txt</code>'),
            ),
            'err_missing_weight' => array(
              'title' => __('Missing weight error', 'woocommerce-freightview'),
              'type' => 'text',
              'description' => __('Error message displayed when a product is missing the weight', 'woocommerce-freightview'),
              'desc_tip' => true,
              'default' => __('Oops! It looks like some of our products are missing a weight.<br/>Please contact our website administrator about the following:', 'woocommerce-freightview')
            )
          );
        }

        /**
         * admin_options function.
         */
        public function admin_options()
        {
          global $woocommerce, $wpdb, $fv_shipping_classes;
          ?>
            <h2><?php _e('Freightview', 'woocommerce');
              ?></h2>

            <input type="hidden" name="woocommerce_freightview_origin_liftgate" value="0"/>
            <input type="hidden" name="woocommerce_freightview_origin_inside" value="0"/>

            <table class="form-table">
              <?php
              $html = '';
              $form_fields = $this->get_form_fields();
              foreach ($form_fields as $k => $v) {
                if (!isset($v['type']) || ($v['type'] == '')) {
                  $v['type'] = 'text'; // Default to "text" field type.
                }

                if (method_exists($this, 'generate_' . $v['type'] . '_html')) {
                  echo $this->{'generate_' . $v['type'] . '_html'}($k, $v);
                } else {
                  echo $this->{'generate_text_html'}($k, $v);
                }

                if ($k == 'api_key'): ?>
                    <tr valign="top">
                        <th scope="row" class="titledesc" style="padding: 0"></th>
                        <td style="padding:0 10px;">
                            <fieldset>
                                <p class="description">
                                  <?php _e('Where is my API key?', 'woocommerce-freightview') ?>
                                    <a href="http://blog.freightview.com/how-to-get-your-freightview-api-key/"
                                       target="_blank">freightview.com</a>
                                    <br/>
                                    <code><?php _e("Choose the 'API Key' from the options menu.", 'woocommerce-freightview') ?></code>
                                </p>
                            </fieldset>
                        </td>
                    </tr>
                <?php endif;
              }
              ?>
            </table>

            <h4><?php _e('Shipping Classes', 'woocommerce-freightview') ?></h4>
            <table class="widefat">
                <thead>
                <tr>
                    <th width="20"></th>
                    <th>
                      <?php _e('Name', 'woocommerce-freightview') ?>
                    </th>
                    <th>
                      <?php _e('Slug', 'woocommerce-freightview') ?>
                    </th>
                </tr>
                </thead>
              <?php
              $classes = get_terms('product_shipping_class', array('hide_empty' => '0'));
              $freightview_classes = array_combine($fv_shipping_classes, $fv_shipping_classes);
              foreach ($classes as $class) {
                $class_no = str_replace('-', '.', str_replace('class-', '', $class->slug));
                unset($freightview_classes[$class_no]);
                echo '<tr><td></td>';
                echo '<td>' . $class->name . '</td>';
                echo '<td>' . $class->slug . '</td>';
                echo '</tr>';
              }
              foreach ($freightview_classes as $fv_class) {
                echo '<tr style="background: #eee">';
                echo '<td><input type="checkbox" name="fv_freight_classes[]" value="' . $fv_class . '" /></td>';
                echo '<td>Class ' . $fv_class . '</td>';
                echo '<td>class-' . str_replace('.', '-', $fv_class) . '</td>';
                echo '</tr>';
              }
              ?>
            </table>
            <input type="submit" name="add_freight_classes" class="button-primary"
                   value="<?php _e('Add Freight Classes', 'woocommerce-freightview') ?>"/>

            <br/>
            <br/>

            <fieldset class="fv">
                <legend><span><?php _e('Send test quote', 'woocommerce-freightview') ?></span></legend>
                <table class="formtable">
                    <tr>
                        <td>
                            <label for="fv_test_pickupDate">
                              <?php _e('Pickup Date', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="text" name="fv_test_pickupDate" id="fv_test_pickupDate"
                                   value="<?php echo date('Y-m-d') ?>"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_originPostalCode">
                              <?php _e('Origin Zip Code', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="text" name="fv_test_originPostalCode" id="fv_test_originPostalCode"
                                   value="83201"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_originType">
                              <?php _e('Origin Type', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <select name="fv_test_originType" id="fv_test_originType">
                                <option value="business dock" selected="selected">business dock</option>
                                <option value="business no dock">business no dock</option>
                                <option value="residential">residential</option>
                                <option value="limited access">limited access</option>
                                <option value="trade show">trade show</option>
                                <option value="construction">construction</option>
                                <option value="farm">farm</option>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_origin_liftgate">
                              <?php _e('Origin Lift Gate?', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="checkbox" name="fv_test_origin_liftgate" id="fv_test_origin_liftgate"
                                   value="1"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_origin_inside">
                              <?php _e('Origin Inside Pickup?', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="checkbox" name="fv_test_origin_inside" id="fv_test_origin_inside" value="1"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_destPostalCode">
                              <?php _e('Destination Zip Code', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="text" name="fv_test_destPostalCode" id="fv_test_destPostalCode" value="83201"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_destType">
                              <?php _e('Destination Type', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <select name="fv_test_destType" id="fv_test_destType">
                                <option value="business dock" selected="selected">business dock</option>
                                <option value="business no dock">business no dock</option>
                                <option value="residential">residential</option>
                                <option value="limited access">limited access</option>
                                <option value="trade show">trade show</option>
                                <option value="construction">construction</option>
                                <option value="farm">farm</option>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_destination_liftgate">
                              <?php _e('Destination Lift Gate?', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="checkbox" name="fv_test_destination_liftgate" id="fv_test_destination_liftgate"
                                   value="1"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_destination_inside">
                              <?php _e('Destination Inside Pickup?', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="checkbox" name="fv_test_destination_inside" id="fv_test_destination_inside"
                                   value="1"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_freightClass">
                              <?php _e('Origin Type', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <select name="fv_test_freightClass" id="fv_test_freightClass">
                                <option value='50' selected='selected'>50</option>
                                <option value='55'>55</option>
                                <option value='60'>60</option>
                                <option value='65'>65</option>
                                <option value='70'>70</option>
                                <option value='77.5'>77.5</option>
                                <option value='85'>85</option>
                                <option value='92.5'>92.5</option>
                                <option value='100'>100</option>
                                <option value='110'>110</option>
                                <option value='125'>125</option>
                                <option value='150'>150</option>
                                <option value='175'>175</option>
                                <option value='200'>200</option>
                                <option value='250'>250</option>
                                <option value='300'>300</option>
                                <option value='400'>400</option>
                                <option value='500'>500</option>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_weight">
                              <?php _e('Weight', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="text" name="fv_test_weight" id="fv_test_weight" value="400"/>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="fv_test_weight">
                              <?php _e('Length x Width x Height (inches)', 'woocommerce-freightview') ?>
                            </label>
                        </td>
                        <td>
                            <input type="text" name="fv_test_width" id="fv_test_width" value="48" size="4"/>x
                            <input type="text" name="fv_test_length" id="fv_test_length" value="48" size="4"/>x
                            <input type="text" name="fv_test_height" id="fv_test_height" value="24" size="4"/>
                        </td>
                    </tr>
                    <tr>
                        <td></td>
                        <td>
                            <input type="submit" class="button-primary"
                                   value="<?php _e('Send Test Quote Request', 'woocommerce-freightview') ?>"
                                   name="fv_test_send"/>
                        </td>
                    </tr>
                </table>
            </fieldset>

            <script><?php include('js/admin.js'); ?></script>

          <?php

        }

        public function process_admin_options()
        {
          global $wpdb;

          $this->validate_settings_fields();

          if (isset($_POST['fv_freight_classes'])) {
            foreach ($_POST['fv_freight_classes'] as $fv_class) {
              wp_insert_term(
                'Class ' . $fv_class, // the term
                'product_shipping_class', // the taxonomy
                array(
                  'slug' => 'class-' . str_replace('.', '-', $fv_class),
                )
              );
            }
          }

          if (isset($_POST['woocommerce_WC_Freightview_default_package_type'])) {
            update_option('woocommerce_WC_Freightview_default_package_type', $_POST['woocommerce_WC_Freightview_default_package_type']);
          }

          if (!$this->admin_options_ran and isset($_POST['fv_test_send'])) {
            $this->admin_options_ran = true;

            $query = array(
              'pickupDate' => $_POST['fv_test_pickupDate'],
              'originPostalCode' => $_POST['fv_test_originPostalCode'],
              'originType' => $_POST['fv_test_originType'],
              'destPostalCode' => $_POST['fv_test_destPostalCode'],
              'destType' => $_POST['fv_test_destType'],
              'items' => array(),
              'charges' => array(),
            );

            if (isset($_POST['fv_test_origin_liftgate'])) {
              $query['charges'][] = 'liftgate pickup';
            }
            if (isset($_POST['fv_test_origin_inside'])) {
              $query['charges'][] = 'inside pickup';
            }
            if (isset($_POST['fv_test_destination_liftgate'])) {
              $query['charges'][] = 'liftgate delivery';
            }
            if (isset($_POST['fv_test_destination_inside'])) {
              $query['charges'][] = 'inside delivery';
            }

            if (count($query['charges']) < 1) {
              unset($query['charges']);
            }

            $query['items'][] = array(
              'freightClass' => (float)$_POST['fv_test_freightClass'],
              'weight' => (int)$_POST['fv_test_weight'],
              'length' => (int)$_POST['fv_test_width'],
              'width' => (int)$_POST['fv_test_length'],
              'height' => (int)$_POST['fv_test_height'],
            );

            if ($this->debug) {
              ppr($query, __('Query', 'woocommerce-freightview'));
              ppr(json_encode($query), __('Query Json', 'woocommerce-freightview'));
            }

            echo '<fieldset class="fv"><legend><span>' . __('Results', 'woocommerce-freightview') . '</span></legend>';
            echo '<div id="fv_results_loading" style="margin-top:20px;padding:20px;">' . __('Loading...', 'woocommerce-freightview') . '</div>';
            $result = json_decode($this->request_estimate($query));

            $byTotal = function ($v) {
              return $v->total;
            };
            $byDays = function ($v) {
              return $v->days;
            };

            if (isset($result->rates)) {
              $result->rates = Sort::comparison($result->rates, array($byDays, $byTotal), array(Sort::ASC, Sort::ASC));
            }

            echo '<table class="widefat">';
            if (is_array($result)) {
              foreach ($result as $k => $v) {
                if (in_array($k, array('items', 'charges', 'rates'))) {
                  continue;
                }
                echo '<tr><th><strong>' . $k . '</strong></th>';
                echo '<td>' . (is_object($v) ? '<pre>' . print_r($v, true) . '</pre>' : $v) . '</td></tr>';
              }
            }
            if (isset($result->charges)) {
              echo '<tr><th><strong>Charges</strong></th><td>' . implode(', ', $result->charges) . '</td></tr>';
            }
            echo '</table>';

            if (isset($result->rates)) {
              echo '<fieldset class="fv"><legend><span>' . __('Rates', 'woocommerce-freightview') . '</span></legend>';
              echo '<table class="widefat">';
              $first = true;
              foreach ($result->rates as $rate) {
                if ($first) {
                  echo '<thead><tr>';
                  foreach ($rate as $k => $v) {
                    if (in_array($k, array())) {
                      continue;
                    }
                    echo '<th>' . $k . '</th>';
                  }
                  echo '</tr></thead>';
                  $first = false;
                }
                echo '<tr>';
                foreach ($rate as $k => $v) {
                  if (in_array($k, array())) {
                    continue;
                  }
                  echo '<td>' . $v . '</td>';
                }
                echo '</tr>';
              }
              echo '</table>';
              echo '</fieldset>';
            }

            echo '</fieldset>';

            ppr(json_encode($result), __('Result Json', 'woocommerce-freightview'));

            echo '<script>document.getElementById("fv_results_loading").style.display = "none";</script>';

            echo '<br/><a href="" class="button-primary">' . __('Back to Settings', 'woocommerce-freightview') . '</a>';

            exit;
          }

          parent::process_admin_options();
        }

        private function request_estimate($query)
        {
          $data_string = json_encode($query);
          FV::log('======> REQUEST : ' . $data_string);

          $ch = curl_init($this->freightview_endpoint);
          curl_setopt($ch, CURLOPT_USERPWD, $this->api_key . ':');
          curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
          curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
          curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
          curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json',
            'Content-Length: ' . strlen($data_string),
            'X-Client-Id: WooCommerce',
          ));

          $result = curl_exec($ch);

          if ((!$result or $result == 'Not Found') and $this->debug == 'yes') {
            $this->log->add('freightview', 'Curl error (' . $this->freightview_endpoint . ')' . curl_error($ch));
          }
          return $result;
        }

        public function checkout_field_update_order_meta($order_id)
        {
          if (isset($_POST) and isset($_POST['shipping_method'])) {
            foreach ($_POST['shipping_method'] as $method) {
              if (strpos($method, self::$static_id) !== false) {
                $ref_ids = str_replace(self::$static_id . '_', '', $method);
                $ref_id_arr = explode('_', $ref_ids);
                $fv_shipment_id = $ref_id_arr[0];
                $fv_shipment_rate_id = $ref_id_arr[1];
              }
            }
            if ($fv_shipment_id) {
              update_post_meta($order_id, 'freightview_shipment_id', sanitize_text_field($fv_shipment_id));
              update_post_meta($order_id, 'freightview_shipment_rate_id', sanitize_text_field($fv_shipment_rate_id));
            }
          }
        }
      }
    }
  }

  add_action('woocommerce_shipping_init', 'freightview_shipping_method_init');

  /*
   * Load frontend assets
   */
  function freightview_frontend_load_assets()
  {
    $plugin_url = plugin_dir_url(__FILE__);
    wp_enqueue_style('freightview_shipping_frontend', $plugin_url . 'css/frontend.css');
    wp_enqueue_script('freightview_shipping_frontend', $plugin_url . 'js/frontend.js', array('jquery'));
  }

  add_action('wp_enqueue_scripts', 'freightview_frontend_load_assets');

  /*
   * Load admin assets
   */
  function freightview_admin_load_css($hook)
  {
    if ($hook != 'woocommerce_page_wc-settings') {
      return;
    }
    $plugin_url = plugin_dir_url(__FILE__);
    wp_enqueue_style('freightview_shipping_admin', $plugin_url . 'css/admin.css');
  }

  add_action('admin_enqueue_scripts', 'freightview_admin_load_css');

  /*
   * Add Freightview to Woocommerce shipping methods
   */
  function add_freightview_shipping_method($methods)
  {
    $methods[] = 'WC_Freightview';

    return $methods;
  }

  add_filter('woocommerce_shipping_methods', 'add_freightview_shipping_method');

  /*
   * add estimate reference numbers to order
   */
  add_action('woocommerce_checkout_update_order_meta', array('WC_Freightview', 'checkout_field_update_order_meta'));

  /*
   * show loading indicator for curl requests
   * woocommerce filters seem to be firing after the curl request, so append to all carts and hide when loaded
   */
  $GLOBALS['loading_indicator_displayed'] = false;
  function fv_filter_the_post($post)
  {
    if (is_cart() and WC()->cart->cart_contents_count > 0 and is_main_query() and !$GLOBALS['loading_indicator_displayed']) {
      ?>
        <div id="estimate_request_loading_indicator" class="entry-content">
            <img src="/wp-admin/images/loading.gif" alt="<?php echo __('Loading...') ?>"/> &nbsp;
          <?php echo __('Processing cart ...');
          ?>
        </div>
        <script>
          document.getElementById("estimate_request_loading_indicator").style.display = 'block';
        </script>
      <?php
      flush();
      $GLOBALS['loading_indicator_displayed'] = true;
    }

    return $post;
  }

  add_filter('the_post', 'fv_filter_the_post');

  /*
   * Remove loading indicator after cart loads
   */
  function fv_action_woocommerce_after_cart_table()
  {
    ?>
      <script>
        var estimate_request_loading_indicator = document.getElementById('estimate_request_loading_indicator');
        if (estimate_request_loading_indicator) {
          estimate_request_loading_indicator.parentNode.removeChild(estimate_request_loading_indicator);
        }
      </script>
    <?php

  }

  add_action('woocommerce_after_cart_table', 'fv_action_woocommerce_after_cart_table');

  /*
   * Add Freightviews quote options to the shipping calculator
   */

  function fv_action_woocommerce_shipping_calculator_enable_postcode()
  {
    global $fv_location_types;
    $showAccessorials = get_option('woocommerce_freightview_settings')['customer_selects_accessorials'];
    if ($showAccessorials === null) {
      $showAccessorials = get_option('woocommerce_WC_Freightview_settings')['customer_selects_accessorials'];
    }
	echo '<!--' . json_encode(array(
			  'showAccessorials' => $showAccessorials,
		  )) . '-->';
    if ($showAccessorials !== "yes") {
      return true;
    }
    $displayFreightViewsOptions = false;

    // check post
    if (isset($_POST['shipping_method'])
      and isset($_POST['shipping_method'][0])
      and strpos($_POST['shipping_method'][0], WC_Freightview_ID) !== false
    ) {
      $displayFreightViewsOptions = true;
    }

    // check session
    if (!isset($_POST['shipping_method'])) {
      $session_shipping_methods = WC()->session->get('chosen_shipping_methods', array());
      foreach ($session_shipping_methods as $method) {
        if (strpos($method, WC_Freightview_ID) !== false) {
          $displayFreightViewsOptions = true;
        }
      }
    }

    $selected_type = WC()->session->get('fv_destination_type', 'business dock');
    $selected_liftgate = WC()->session->get('fv_destination_liftgate');
    $selected_inside = WC()->session->get('fv_destination_inside');

    echo '<!--' . json_encode(array(
        'showAccessorials' => $showAccessorials,
        'custom' => WC()->session->get('chosen_shipping_methods', array()),
        'displayFreightViewsOptions' => $displayFreightViewsOptions,
        'destination_type' => $selected_type,
        'destination_liftgate' => $selected_liftgate,
        'destination_inside' => $selected_inside
      )) . '-->';
    ?>

      <div class="delivery_options" id="FV_Delivery_Options"
           style="display: <?php echo $displayFreightViewsOptions ? 'block' : 'none' ?>">
          <p class="form-row form-row-wide form-freightview-select">
              <label for="cart_destination_type">Destination ship type</label>
              <select name="destination_type" id="cart_destination_type" placeholder="Ship Type"
                      class="freightview-select">
                <?php
                foreach ($fv_location_types as $name => $label) {
                  $selected = $name == $selected_type ? ' selected="selected"' : '';
                  echo '<option value="' . $name . '" ' . $selected . '>' . $label . '</option>';
                }
                ?>
              </select>
          </p>
          <p class="form-row form-row-wide form-freightview-checkbox freightview-checkbox-liftgate">
              <input type="hidden" name="destination_liftgate" value="0"/>
              <label for="cart_destination_liftgate">
                  <input type="checkbox" name="destination_liftgate" id="cart_destination_liftgate"
                         class="freightview-checkbox"
                         value="1" <?php echo $selected_liftgate == 1 ? 'checked="checked"' : '' ?> />
                <?php _e('Lift gate required for delivery?', 'woocommerce-freightview') ?>
              </label>
          </p>
          <p class="form-row form-row-wide form-freightview-checkbox freightview-checkbox-dest-inside"
             class="freightview-checkbox">
              <input type="hidden" name="destination_inside" value="0"/>
              <label for="cart_destination_inside">
                  <input type="checkbox" name="destination_inside" id="cart_destination_inside"
                         class="freightview-checkbox"
                         value="1" <?php echo $selected_inside == 1 ? 'checked="checked"' : '' ?> />
                <?php _e('Inside delivery required?', 'woocommerce-freightview') ?>
              </label>
          </p>
      </div>
    <?php
    return true;
  }

  add_action('woocommerce_shipping_calculator_enable_postcode', 'fv_action_woocommerce_shipping_calculator_enable_postcode');

  /*
   * Add freightview quote options to the checkout form
   */
  function custom_override_checkout_fields($fields)
  {
    FV::log('custom_override_checkout_fields');
    global $fv_location_types;
    $showAccessorials = get_option('woocommerce_freightview_settings')['customer_selects_accessorials'];
    if ($showAccessorials === null) {
	    $showAccessorials = get_option('woocommerce_WC_Freightview_settings')['customer_selects_accessorials'];
    }

    if ($showAccessorials !== "yes") {
      return $fields;
    }

    $fields['order']['destination_type'] = array(
      'label' => __('Delivery Type', 'woocommerce'),
      'placeholder' => _x('business dock', 'placeholder', 'woocommerce'),
      'default' => WC()->session->get('fv_destination_type', 'business dock'),
      'required' => true,
      'class' => array('form-row-wide', 'address-field'),
      'clear' => true,
      'type' => 'select',
      'options' => $fv_location_types,
    );

    $fields['order']['destination_liftgate'] = array(
      'label' => __('Lift gate required for delivery?', 'woocommerce'),
      'default' => WC()->session->get('fv_destination_liftgate', 0),
      'class' => array('form-row-wide', 'address-field', 'input-checkbox', 'validate-required'),
      'clear' => true,
      'type' => 'checkbox',
    );

    $fields['order']['destination_inside'] = array(
      'label' => __('Inside delivery required?', 'woocommerce'),
      'default' => WC()->session->get('fv_destination_inside', 0),
      'class' => array('form-row-wide', 'address-field', 'input-checkbox', 'validate-required'),
      'clear' => true,
      'type' => 'checkbox',
    );

    return $fields;
  }

  add_filter('woocommerce_checkout_fields', 'custom_override_checkout_fields');

  /*
   * Add the values from the shipping calculator to the actual calculations
   */
  function fv_filter_woocommerce_cart_shipping_packages($packages)
  {
    $showAccessorials = get_option('woocommerce_freightview_settings')['customer_selects_accessorials'];
    if ($showAccessorials === null) {
      $showAccessorials = get_option('woocommerce_WC_Freightview_settings')['customer_selects_accessorials'];
    }
    if ($showAccessorials !== "yes") {
      return $packages;
    }

    if (count($packages)) {
      $d = $packages[0]['destination'];

      if (isset($_POST['destination_type'])) {
        WC()->session->set('fv_destination_type', $_POST['destination_type']);
      }
      $d['destination_type'] = WC()->session->get('fv_destination_type', 'business dock');

      if (isset($_POST['destination_liftgate'])) {
        WC()->session->set('fv_destination_liftgate', (int)$_POST['destination_liftgate']);
      }
      $d['destination_liftgate'] = WC()->session->get('fv_destination_liftgate', 0);

      if (isset($_POST['destination_inside'])) {
        WC()->session->set('fv_destination_inside', (int)$_POST['destination_inside']);
      }
      $d['destination_inside'] = WC()->session->get('fv_destination_inside', 0);

      FV::log($d, 'fv_filter_woocommerce_cart_shipping_packages');
      $packages[0]['destination'] = $d;
    }

    return $packages;
  }

  add_filter('woocommerce_cart_shipping_packages', 'fv_filter_woocommerce_cart_shipping_packages');

  /*
   * Add the values from the checkout form to the actual calculations
   */
  function fv_action_woocommerce_checkout_update_order_review($post)
  {
    FV::log('fv_action_woocommerce_checkout_update_order_review');
    $post_fields = array();
    parse_str($post, $post_fields);

    $destination_type = isset($post_fields['destination_type']) ? $post_fields['destination_type'] : '';
    $destination_liftgate = isset($post_fields['destination_liftgate']) ? (int)$post_fields['destination_liftgate'] : 0;
    $destination_inside = isset($post_fields['destination_inside']) ? (int)$post_fields['destination_inside'] : 0;

    WC()->session->set('fv_destination_type', $destination_type);
    WC()->session->set('fv_destination_liftgate', $destination_liftgate);
    WC()->session->set('fv_destination_inside', $destination_inside);
  }

  add_action('woocommerce_checkout_update_order_review', 'fv_action_woocommerce_checkout_update_order_review');


  // define the item in the meta box by adding an item to the $actions array
  function add_order_meta_box_actions($actions)
  {
    $actions['ns_freightview_book'] = __('Book this shipment on Freightview', 'woocommerce-freightview');
    return $actions;
  }

  // add our own item to the order actions meta box
  add_action('woocommerce_order_actions', 'add_order_meta_box_actions');

  function build_freightview_url($fv_ids)
  {
    return "https://www.freightview.com/app/shipment/" . $fv_ids['shipment_id'] . "/select-rate/" . $fv_ids['rate_id'];
  }

  function get_freightview_ids($postId)
  {
    return array(
      'shipment_id' => get_post_meta($postId, 'freightview_shipment_id', true),
      'rate_id' => get_post_meta($postId, 'freightview_shipment_rate_id', true)
    );
  }

  // run the code that should execute with this action is triggered
  function process_order_meta_box_actions($order)
  {
    // we have the current $order object from the edit order page that we're currently on and can do additional things with it
    $fv_ids = get_freightview_ids($order->id);

    if (!empty($fv_ids['shipment_id']) && !empty($fv_ids['rate_id'])) {
      header('Location: ' + build_freightview_url($fv_ids));
    }
  }

  // process the custom order meta box order action
  add_action('woocommerce_order_action_ns_freightview_book', 'process_order_meta_box_actions');

  function freightview_woocommerce_actions_js($postId)
  {
    $fv_ids = get_freightview_ids($postId);
    ?>
      <script type='text/javascript'>
        jQuery(document).ready(function ($) {
          $('#actions button.wc-reload').click(function (e) {
            if ($(this).parents('#actions').find('[name="wc_order_action"]').val() == "ns_freightview_book") {
              e.preventDefault();
              window.open("<?php echo build_freightview_url($fv_ids); ?>", 'newtab');
            }
          });
        });
      </script>
    <?php
  }

  add_action('woocommerce_order_actions_start', 'freightview_woocommerce_actions_js');

  function freightview_woocommerce_shipping_packages()
  {
    return array(
      'Pallets_other' => 'Pallets_other',
      'Bags' => 'Bags',
      'Bales' => 'Bales',
      'Boxes' => 'Boxes',
      'Bundles' => 'Bundles',
      'Carpets' => 'Carpets',
      'Coils' => 'Coils',
      'Crates' => 'Crates',
      'Cylinders' => 'Cylinders',
      'Drums' => 'Drums',
      'Pails' => 'Pails',
      'Reels' => 'Reels',
      'Rolls' => 'Rolls',
      'TubesPipes' => 'TubesPipes',
      'Slipsheets' => 'Slipsheets',
      'Unit' => 'Unit'
    );
  }

  function freightview_woocommerce_default_shipping_package()
  {
    return get_option('woocommerce_WC_Freightview_default_package_type', 'Pallets_other');
  }

  function freightview_woocommerce_product_options_shipping_package()
  {

    $value = get_post_meta(get_the_ID(), '_fv_wc_product_shipping_package', true);
    $value = $value ? $value : freightview_woocommerce_default_shipping_package();

    woocommerce_wp_select(array(
      'id' => '_fv_wc_product_shipping_package',
      'label' => __('Package', 'woocommerce'),
      'options' => freightview_woocommerce_shipping_packages(),
      'value' => $value,
      'description' => 'Choose the correct packaging type.',
      'desc_tip' => 'true',
    ));

  }

  add_action('woocommerce_product_options_shipping', 'freightview_woocommerce_product_options_shipping_package');

  function freightview_woocommerce_product_save_custom_fields($post_id)
  {
    if (!empty($_POST['_fv_wc_product_shipping_package'])) {
      update_post_meta($post_id, '_fv_wc_product_shipping_package', esc_attr($_POST['_fv_wc_product_shipping_package']));
    }

  }

  add_action('woocommerce_process_product_meta', 'freightview_woocommerce_product_save_custom_fields');
}
